import software.amazon.awscdk.Stack;
import software.amazon.awscdk.StackProps;
import software.amazon.awscdk.services.dynamodb.*;
import software.amazon.awscdk.services.dynamodb.CfnGlobalTable.GlobalSecondaryIndexProperty;
import software.amazon.awscdk.services.dynamodb.CfnGlobalTable.ReplicaGlobalSecondaryIndexSpecificationProperty;
import software.constructs.Construct;

import java.util.List;


public class CdkCFNStack extends Stack {
  public CdkCFNStack(final Construct scope, final String id) {
    this(scope, id, null);
  }

  public CdkCFNStack(final Construct scope, final String id, final StackProps props) {
    super(scope, id, props);

    CfnGlobalTable cfnGlobalTable = CfnGlobalTable.Builder.create(this, "TestCdkCfn")
        .tableName("TestTable")
        .writeProvisionedThroughputSettings(CfnGlobalTable.WriteProvisionedThroughputSettingsProperty.builder()
            .writeCapacityAutoScalingSettings(CfnGlobalTable.CapacityAutoScalingSettingsProperty.builder()
                .maxCapacity(10)
                .minCapacity(1)
                .targetTrackingScalingPolicyConfiguration(CfnGlobalTable.TargetTrackingScalingPolicyConfigurationProperty.builder()
                    .targetValue(75)
                    .build())
                .build())
            .build())
        .attributeDefinitions(List.of(CfnGlobalTable.AttributeDefinitionProperty.builder()
                .attributeName("pk")
                .attributeType("S")
                .build(),
            CfnGlobalTable.AttributeDefinitionProperty.builder()
                .attributeName("sk")
                .attributeType("S")
                .build()))
        .keySchema(List.of(CfnGlobalTable.KeySchemaProperty.builder()
                .attributeName("pk")
                .keyType("HASH")
                .build(),
            CfnGlobalTable.KeySchemaProperty.builder()
                .attributeName("sk")
                .keyType("RANGE")
                .build()))
        .billingMode("PROVISIONED")
        .streamSpecification(CfnGlobalTable.StreamSpecificationProperty.builder()
            .streamViewType("KEYS_ONLY")
            .build())
        .globalSecondaryIndexes(List.of(GlobalSecondaryIndexProperty.builder()
            .indexName("FirstGSI")
            .keySchema(List.of(CfnGlobalTable.KeySchemaProperty.builder()
                .attributeName("sk")
                .keyType("HASH")
                .build(), CfnGlobalTable.KeySchemaProperty.builder()
                .attributeName("pk")
                .keyType("RANGE")
                .build()))
            .projection(CfnGlobalTable.ProjectionProperty.builder()
                .projectionType("KEYS_ONLY")
                .build())
            .writeProvisionedThroughputSettings(CfnGlobalTable.WriteProvisionedThroughputSettingsProperty.builder()
                .writeCapacityAutoScalingSettings(CfnGlobalTable.CapacityAutoScalingSettingsProperty.builder()
                    .maxCapacity(22)
                    .minCapacity(6)
                    .targetTrackingScalingPolicyConfiguration(CfnGlobalTable.TargetTrackingScalingPolicyConfigurationProperty.builder()
                        .targetValue(82)
                        .build())
                    .build())
                .build())
            .build()))
        .replicas(List.of(
            CfnGlobalTable.ReplicaSpecificationProperty.builder()
                .region("eu-west-1")
                .globalSecondaryIndexes(List.of(ReplicaGlobalSecondaryIndexSpecificationProperty.builder()
                    .indexName("FirstGSI")
                    .readProvisionedThroughputSettings(CfnGlobalTable.ReadProvisionedThroughputSettingsProperty.builder()
                        .readCapacityAutoScalingSettings(CfnGlobalTable.CapacityAutoScalingSettingsProperty.builder()
                            .maxCapacity(20)
                            .minCapacity(4)
                            .targetTrackingScalingPolicyConfiguration(CfnGlobalTable.TargetTrackingScalingPolicyConfigurationProperty.builder()
                                .targetValue(82)
                                .build())
                            .build())
                        .build())
                    .build()))
                .readProvisionedThroughputSettings(CfnGlobalTable.ReadProvisionedThroughputSettingsProperty.builder()
                    .readCapacityAutoScalingSettings(CfnGlobalTable.CapacityAutoScalingSettingsProperty.builder()
                        .maxCapacity(20)
                        .minCapacity(5)
                        .targetTrackingScalingPolicyConfiguration(CfnGlobalTable.TargetTrackingScalingPolicyConfigurationProperty.builder()
                            .targetValue(65)
                            .build())
                        .build())
                    .build())
                .build(), CfnGlobalTable.ReplicaSpecificationProperty.builder()
                .region("eu-north-1")
                .globalSecondaryIndexes(List.of(ReplicaGlobalSecondaryIndexSpecificationProperty.builder()
                    .indexName("FirstGSI")
                    .readProvisionedThroughputSettings(CfnGlobalTable.ReadProvisionedThroughputSettingsProperty.builder()
                        .readCapacityAutoScalingSettings(CfnGlobalTable.CapacityAutoScalingSettingsProperty.builder()
                            .maxCapacity(20)
                            .minCapacity(4)
                            .targetTrackingScalingPolicyConfiguration(CfnGlobalTable.TargetTrackingScalingPolicyConfigurationProperty.builder()
                                .targetValue(82)
                                .build())
                            .build())
                        .build())
                    .build()))
                .readProvisionedThroughputSettings(CfnGlobalTable.ReadProvisionedThroughputSettingsProperty.builder()
                    .readCapacityAutoScalingSettings(CfnGlobalTable.CapacityAutoScalingSettingsProperty.builder()
                        .maxCapacity(20)
                        .minCapacity(5)
                        .targetTrackingScalingPolicyConfiguration(CfnGlobalTable.TargetTrackingScalingPolicyConfigurationProperty.builder()
                            .targetValue(65)
                            .build())
                        .build())
                    .build())
                .build()))
        .build();


  }
}
